
/********************* UNITS OF MEASURE (Units.h) ***************************/

#ifndef _LINANDANG_UNITS_H
#define _LINANDANG_UNITS_H
#include <math.h>
#ifndef M_PI
#define M_PI         3.14159265358979323846
#endif
#ifndef M_PI_2
#define M_PI_2       1.57079632679489661923
#endif
 
#define ONEPI           (M_PI)
#define HALFPI          (M_PI_2)
#define TWOPI           (M_PI+M_PI)

typedef enum
{	UNDEFINEDLINEAR_UNITS,
	NMILES, 
	MILES, 
	YARDS, 
	FEET, 
	INCHES, 
	FATHOMS, 
	LEAGUES, 
	ARCSECS,
	KILOMETERS,
	METERS,
	CENTIMETERS,
	UNKNOWNLINEAR_UNITS,
	UNDEFINEDANGULAR_UNITS,
	DEGREES, 
	MINUTES,
	SECONDS, 
	RADIANS,
	DDDMMSS,
	UNKNOWNANGULAR_UNITS
}Units;
typedef Units LinearUnit;
typedef Units AngularUnit;

#define UNDEFINED_UNITS		UNDEFINEDLINEAR_UNITS
#define NO_UNITS		UNDEFINEDLINEAR_UNITS

#define DegreesToRadians(d) 	( UnitsConvert(DEGREES,RADIANS)*(d) )
#define RadiansToDegrees(r) 	( UnitsConvert(RADIANS,DEGREES)*(r) )
#define UnitsAreLinear(units)	\
	( (units) > UNDEFINEDLINEAR_UNITS && (units) < UNKNOWNLINEAR_UNITS )
#define UnitsAreAngular(units)	\
	( (units) > UNDEFINEDANGULAR_UNITS && (units) < UNKNOWNANGULAR_UNITS )
#define UnitsAreDefined(units)	\
	( UnitsAreLinear(units) || UnitsAreAngular(units) )
#define UnitsTwoPi(units)	\
	( !UnitsAreAngular(units) ? 0.0 : TWOPI*UnitsConvert(RADIANS,units) )
#define UnitsHalfPi(units)	\
	( !UnitsAreAngular(units) ? 0.0 : HALFPI*UnitsConvert(RADIANS,units) )

#ifdef __STDC__
extern	char*		UnitsEncode( Units units, char* label );
extern	Units		UnitsDecode( char* label ); 
extern	double		UnitsConvert( Units from,Units to );
#elif defined MS_DOS
extern	char*		UnitsEncode( Units units, char* label );
extern	Units		UnitsDecode( char* label ); 
extern	double		UnitsConvert( Units from,Units to );
#else
extern	char*		UnitsEncode();
extern	Units		UnitsDecode();
extern	double		UnitsConvert();
#endif

#endif
