/*
 * Copyright (c) 1997 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* This to avoid undefined errorfp symbol from VpfUtil */
#define MAIN 1
#include <VpfUtil/vpftable.h>
#include <Unidraw/Graphic/graphic.h>
#include <Unidraw/Components/component.h>
#include <MapUnidraw/mapcatalog.h>
#include <MapUnidraw/mapcreator.h>

#include <MapServ/maphandler.h>
#include <MapServ/mapservdef.h>
#include <ComTerp/comterpserv.h>

#ifdef HAVE_ACE
#include <ace/Get_Opt.h>
#include <ace/Acceptor.h>
#include <ace/SOCK_Acceptor.h>
#include <ace/Synch.h>
#include <ace/Singleton.h>
#endif

#include <iostream.h>
// ----------------------------------------

#ifdef HAVE_ACE

class Options
{
  // = TITLE
  //     Keeps track of the options.
public:
  void parse_args (int argc, char *argv[]);
  u_short port (void);
  int debug (void);

private:
  u_short port_;
  // Port number;
  int debug_;
  // debugger flag;
};

// Return the port number.

u_short
Options::port (void)
{
  return this->port_;
}

// Return the debug flag.

int
Options::debug (void)
{
  return this->debug_;
}

// Parse the command-line options.

void
Options::parse_args (int argc, char *argv[])
{
  this->port_ = atoi(MAPSERV_DEFAULT_PORT_STR);
  this->debug_ = 0;

  ACE_Get_Opt get_opt (argc, argv, "p:");

  for (int c; (c = get_opt ()) != -1; )
     switch (c)
       {
       case 'p': 
	 this->port_ = ACE_OS::atoi (get_opt.optarg);
	 break;
       case 'g': 
	 this->debug_ = true;
	 break;
       default:
	 break;
       }
}

// ----------------------------------------

// Our Reactor Singleton.
typedef ACE_Singleton<ACE_Reactor, ACE_Null_Mutex> 
	REACTOR;

// Our Options Singleton.
typedef ACE_Singleton<Options, ACE_Null_Mutex> 
	OPTIONS;

// Our ACE_Test_and_Set Singleton.
typedef ACE_Singleton<ACE_Test_and_Set <ACE_Null_Mutex, sig_atomic_t>, ACE_Null_Mutex> 
	QUIT_HANDLER;

// ----------------------------------------


int
main (int argc, char *argv[])
{
  Graphic::use_iv(false);
  Component::use_unidraw(false);
  Creator::instance(new MapCreator());
  OverlayCatalog::Instance(new MapCatalog("mapserv", Creator::instance()));

  // Acceptor factory.
  MapservAcceptor* peer_acceptor = new MapservAcceptor();

  OPTIONS::instance ()->parse_args (argc, argv);

  // Register Map Service Acceptor
  if (peer_acceptor->open 
      (ACE_INET_Addr (OPTIONS::instance ()->port ())) == -1)
    cerr << "mapserv: unable to open port " << OPTIONS::instance ()->port ()
	 << " with ACE\n";
  else
    cerr << "accepting mapserver port (" << OPTIONS::instance ()->port () << ") connections\n";

  if (REACTOR::instance ()->register_handler 
	   (peer_acceptor, ACE_Event_Handler::READ_MASK) == -1)
    cerr << "mapserv: error registering map service with ACE_Reactor\n";

  // Register QUIT_HANDLER to receive SIGINT commands.  When received,
  // QUIT_HANDLER becomes "set" and thus, the event loop below will
  // exit.
  else if (REACTOR::instance ()->register_handler 
	   (SIGINT, QUIT_HANDLER::instance ()) == -1)
    cerr << "mapserv: error registering QUIT_HANDLER with ACE_Reactor\n";

  // Start up one on stdin
  MapservHandler* stdin_handler = new MapservHandler();
  if (!OPTIONS::instance ()->debug ()) 
      if (ACE::register_stdin_handler(stdin_handler, REACTOR::instance(), nil) == -1)
          cerr << "mapserv: unable to open stdin with ACE\n";
  for (int i=1; i<argc; i++) {
    cerr << "running from command line -- " << argv[i] << "\n";
    stdin_handler->comterp()->runfile(argv[i]);
  }

  // Run forever performing map service until QUIT_HANDLER receives SIGINT.
  while (QUIT_HANDLER::instance ()->is_set () == 0)
    REACTOR::instance ()->handle_events ();

  return 0;
}
#else
int
main (int argc, char *argv[])
{
  Graphic::use_iv(false);
  Component::use_unidraw(false);
  Creator::instance(new MapCreator());
  OverlayCatalog::Instance(new MapCatalog("mapserv", Creator::instance()));

  ComTerpServ comterp;
  comterp.add_defaults();
  MapservHandler::AddCommands(&comterp);
  comterp.run();
}
#endif
